@@ -61,11 +61,11 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
   d <- length(dd) # d >= 2
   d.seq <- 1:(d-1)
   n <- dd[d]
-
+  
   x.tnsr <- as.tensor(x)
   tnsr.norm <- fnorm(x.tnsr)
   eigen.gap <- array(NA, c(d-1,max(dd[-d]),maxiter))
-
+  
   if(method=="TIPUP"){
     ans.init <- tipup.init.tensor(x,r,h0,norm.true=TRUE)
   }else if(method=="TOPUP"){
@@ -81,16 +81,16 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
   dis <- 1
   fnorm.resid <- rep(0,maxiter)
   #x.hat <- get.hat(x.tnsr,ans.init$Q,d.seq)
-
+  
   x.hat <- ttl(x.tnsr,lapply(ans.init$Q,t),d.seq)
   x.hat <- ttl(x.hat,ans.init$Q,d.seq)
-
+  
   fnorm.resid[1] <- fnorm(x.tnsr-x.hat)/tnsr.norm
   ans.Q <- ans.init$Q
   Ft <- ttl(x.tnsr,lapply(ans.Q,t),d.seq)
   Ft.all <- apply(Ft@data,c(1:(d-1)),sum)
   fnorm.resid[iiter+1] <- fnorm(x.tnsr-x.hat)/tnsr.norm
-
+  
   if(iter==TRUE){
     while((dis > tol) & (iiter < maxiter)){
       for(i in 1:(d-1)){
@@ -106,7 +106,7 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
         }
         ddd=dd[-d]
         eigen.gap[i,1:dd[i],1+iiter]=ans.iter$lambda[[1]]
-
+        
         if(vmax==TRUE){
           for(j in 1:(d-1)){
             if(r[j] > 1){
@@ -120,14 +120,14 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
             }
           }
         }
-
+        
         #x.hat <- get.hat(x.tnsr,ans.Q,d.seq)
         x.hat <- ttl(x.tnsr,lapply(ans.Q,t),d.seq)
         x.hat <- ttl(x.hat,ans.Q,d.seq)
-
+        
         Ft <- ttl(x.tnsr,lapply(ans.Q,t),d.seq)
         Ft.all <- apply(Ft@data,c(1:(d-1)),sum)
-
+        
         fnorm.resid[iiter+1] <- fnorm(x.tnsr-x.hat)/tnsr.norm
         dis <- abs(fnorm.resid[iiter+1] - fnorm.resid[iiter])
         if(iiter==1){
@@ -143,6 +143,10 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
   eigen.gap[,,maxiter]=eigen.gap[,,iiter-1]
   fnorm.resid <- fnorm.resid[fnorm.resid != 0]
   norm.percent <- c(ans.init$norm.percent,fnorm.resid[1],tail(fnorm.resid,1))
+  x0 <- matrix(x,prod(dd[-d]))
+  x0 <- t(scale(t(x0),scale=FALSE) )
+  x0 <- array(x0,dd)
+  rsquare <- 1- fnorm(x.tnsr-x.hat)^2/fnorm(as.tensor(x0))^2
   return(list("Ft"=aperm(Ft@data,c(d,1:(d-1))),"Ft.all"=Ft.all,"Q"=ans.Q,"x.hat"=aperm(x.hat@data,c(d,1:(d-1))),"niter"=iiter,"fnorm.resid"=fnorm.resid[iiter]))
 }
 
@@ -152,7 +156,7 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
 #' Function for rank determination of tensor factor models with Tucker Structure.
 #' Two unfolding methods of the auto-covariance tensor, Time series Outer-Product Unfolding Procedure (TOPUP), Time series Inner-Product Unfolding Procedure (TIPUP),
 #' are included, as determined by the value of \code{method}.
-#' Different penalty functions for the information criterion (BIC) and the eigen ratio criterion (ER) can be used,
+#' Different penalty functions for the information criterion (IC) and the eigen ratio criterion (ER) can be used,
 #' which should be specified by the value of \code{rank} and \code{penalty}. The information criterion resembles BIC in the vector factor model literature.
 #' And the eigen ratio criterion is similar to the eigenvalue ratio based methods in the vector factor model literature.
 #'@details
@@ -171,14 +175,14 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
 #'@name tenFM.rank
 #'@rdname tenFM.rank
 #'@aliases tenFM.rank
-#'@usage tenFM.rank(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,penalty=1,
+#'@usage tenFM.rank(x,r,h0=1,rank='IC',method='TIPUP',inputr=FALSE,iter=TRUE,penalty=1,
 #'delta1=0,tol=1e-5,maxiter=100)
 #'@export
 #'@param x \eqn{T \times d_1 \times \cdots \times d_K} tensor-valued time series.
 #'@param r input rank of the factor tensor.
 #'@param h0 the number of lags used in auto-covariance tensor.
 #'@param rank character string, specifying the type of the rank determination method to be used. \describe{
-#'  \item{\code{"BIC",}}{information criterion.}
+#'  \item{\code{"IC",}}{information criterion.}
 #'  \item{\code{"ER",}}{eigen ratio criterion.}
 #'}
 #'@param method character string, specifying the type of the factor estimation method to be used. \describe{
@@ -186,23 +190,21 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
 #'  \item{\code{"TOPUP",}}{TOPUP method.}
 #'}
 #'@param inputr boolean, if TRUE, use input rank for each iteration; if FLASE, update the rank r in each iteration.
-#'@param iter boolean, specifying using an iterative approach or an non-iterative approach.
-#'@param penalty takes value in {1,2,3,4,5}, decide which penalty function to use for each tesnor mode \eqn{k}. Here \eqn{\nu} is a tuning parameter defined in the argument "\code{delta1}", and \eqn{d=\prod_{i=1}^{K} d_k }.
-#'  \describe{
-#'  \item{}{When \code{rank}= '\code{BIC}':}
-#'  \item{}{if \code{penalty}=1, \eqn{g_1= \frac{h_0 d^{2-2\nu}}{T}\log(\frac{dT}{d+T})};}
-#'  \item{}{if \code{penalty}=2, \eqn{g_2= h_0 d^{2-2\nu}(\frac{1}{T}+\frac{1}{d})\log(\frac{dT}{d+T})};}
-#'  \item{}{if \code{penalty}=3, \eqn{g_3= \frac{h_0 d^{2-2\nu}}{T} \log(\min{(d,T)})};}
-#'  \item{}{if \code{penalty}=4, \eqn{g_4= h_0 d^{2-2\nu}(\frac{1}{T}+\frac{1}{d})\log(\min{(d,T)})};}
-#'  \item{}{if \code{penalty}=5, \eqn{g_5= h_0 d^{2-2\nu}(\frac{1}{T}+\frac{1}{d})\log(\min{(d_k,T)})}.}
-#'  \item{}{When \code{rank}= '\code{ER}':}
-#'  \item{}{if \code{penalty}=1, \eqn{h_1= c_0 h_0};}
-#'  \item{}{if \code{penalty}=2, \eqn{h_2= \frac{h_0 d^2}{T^2}};}
-#'  \item{}{if \code{penalty}=3, \eqn{h_3= \frac{h_0 d^2}{T^2 d_k^2}};}
-#'  \item{}{if \code{penalty}=4, \eqn{h_4= \frac{h_0 d^2}{T^2 d_k^2} + \frac{h_0 d_k^2}{T^2}};}
-#'  \item{}{if \code{penalty}=5, \eqn{h_5= \frac{h_0 d^2}{T^2 d_k^2} + \frac{h_0 dd_k^2}{T^2}}.}
-#'}
-#'@param delta1 weakest factor strength, a tuning parameter used for BIC method only, default value is 0.
+#'@param iter boolean, specifying using an iterative approach or a non-iterative approach.
+#'@param penalty takes value in {1,2,3,4,5}, decides which penalty function to use for each tesnor mode \eqn{k}. Here \eqn{\nu} is a tuning parameter defined in the argument "\code{delta1}", and \eqn{d=\prod_{i=1}^{K} d_k }. \cr
+#' When \code{rank}= '\code{IC}':\cr
+#' if \code{penalty}=1, \eqn{g_1= \frac{h_0 d^{2-2\nu}}{T}\log(\frac{dT}{d+T})};\cr
+#' if \code{penalty}=2, \eqn{g_2= h_0 d^{2-2\nu}(\frac{1}{T}+\frac{1}{d})\log(\frac{dT}{d+T})};\cr
+#' if \code{penalty}=3, \eqn{g_3= \frac{h_0 d^{2-2\nu}}{T} \log(\min{(d,T)})};\cr
+#' if \code{penalty}=4, \eqn{g_4= h_0 d^{2-2\nu}(\frac{1}{T}+\frac{1}{d})\log(\min{(d,T)})};\cr
+#' if \code{penalty}=5, \eqn{g_5= h_0 d^{2-2\nu}(\frac{1}{T}+\frac{1}{d})\log(\min{(d_k,T)})}.\cr
+#' When \code{rank}= '\code{ER}':\cr
+#' if \code{penalty}=1, \eqn{h_1= c_0 h_0};\cr
+#' if \code{penalty}=2, \eqn{h_2= \frac{h_0 d^2}{T^2}};\cr
+#' if \code{penalty}=3, \eqn{h_3= \frac{h_0 d^2}{T^2 d_k^2}};\cr
+#' if \code{penalty}=4, \eqn{h_4= \frac{h_0 d^2}{T^2 d_k^2} + \frac{h_0 d_k^2}{T^2}};\cr
+#' if \code{penalty}=5, \eqn{h_5= \frac{h_0 d^2}{T^2 d_k^2} + \frac{h_0 dd_k^2}{T^2}}.\cr
+#'@param delta1 weakest factor strength, a tuning parameter used for IC method only, default value is 0.
 #'@param tol tolerance in terms of the Frobenius norm.
 #'@param maxiter maximum number of iterations if error stays above \code{tol}.
 #'@return return a list containing the following:\describe{
@@ -219,7 +221,7 @@ tenFM.est=function(x,r,h0=1,method='TIPUP',iter=TRUE,vmax=FALSE,tol=1e-5,maxiter
 #'lambda <- sqrt(prod(dims))
 #'x <- tenFM.sim(Ft,dims=dims,lambda=lambda,A=NULL,cov='iid') # generate t*dims tensor time series
 #'rank <- tenFM.rank(x,c(4,4,4),h0=1,rank='ER',method='TIPUP')  # Estimate the rank
-tenFM.rank = function(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,penalty=1,delta1=0,tol=1e-5,maxiter=100){
+tenFM.rank = function(x,r,h0=1,rank='IC',method='TIPUP',inputr=FALSE,iter=TRUE,penalty=1,delta1=0,tol=1e-5,maxiter=100){
   #x <- aperm(x@data,c(2:length(dim(x)),1))
   x <- aperm(x,c(2:length(dim(x)),1))
   dd <- dim(x)
@@ -229,8 +231,8 @@ tenFM.rank = function(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,
   x.tnsr <- as.tensor(x)
   tnsr.norm <- fnorm(x.tnsr)
   factor.num <- array(NA, c(d-1,5,maxiter))
-  eigen.gap <- array(NA, c(d-1,max(dd[-d]),maxiter ))
-
+  eigen.gap <- array(NA, c(d-1,max(dd[-d]),maxiter+1))
+  
   if(method=="TIPUP"){
     ans.init <- tipup.init.tensor(x,r,h0,norm.true=TRUE)
   }else if(method=="TOPUP"){
@@ -241,13 +243,13 @@ tenFM.rank = function(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,
   }
   ddd=dd[-d]
   for(i in 1:(d-1)){
-    if(rank=='BIC'){
+    if(rank=='BIC'|rank=='IC'){
       sigmas.hat=1
       factor.num[i,,1]=tensor.bic(ans.init$lambda[[i]]/sigmas.hat,h0,ddd[i],ddd[-i],n,delta1)
     }else if(rank=='ER'){
       factor.num[i,,1]=tensor.ratio(ans.init$lambda[[i]],ddd[i],ddd[-i],n)
     }else{
-      stop('Wrong rank method !')  # should catch exception
+      stop('Wrong rank method !')
     }
     eigen.gap[i,1:dd[i],1]=ans.init$lambda[[i]]
   }
@@ -257,10 +259,10 @@ tenFM.rank = function(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,
   #x.hat <- get.hat(x.tnsr,ans.init$Q,d.seq)
   x.hat <- ttl(x.tnsr,lapply(ans.init$Q,t),d.seq)
   x.hat <- ttl(x.hat,ans.init$Q,d.seq)
-
+  
   fnorm.resid[1] <- fnorm(x.tnsr-x.hat)/tnsr.norm
   ans.Q <- ans.init$Q
-
+  
   if(iter==TRUE){
     while((dis > tol) & (iiter < maxiter)){
       for(i in 1:(d-1)){
@@ -272,27 +274,27 @@ tenFM.rank = function(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,
           ans.iter <- topup.init.tensor(x.new,c(r[i],r[-i]),h0,oneside.true=TRUE,norm.true=FALSE)
           ans.Q[[i]] <- ans.iter$Q[[1]]
         }else{
-          stop('Wrong estimation method input !')  # should catch exception
+          stop('Wrong estimation method input !')
         }
         ddd=dd[-d]
-
-        if(rank=='BIC'){
+        
+        if(rank=='BIC'|rank=='IC'){
           factor.num[i,,1+iiter]=tensor.bic(ans.iter$lambda[[1]]/sigmas.hat,h0,ddd[i],ddd[-i],n,delta1)
         }else if(rank=='ER'){
           factor.num[i,,1+iiter]=tensor.ratio(ans.iter$lambda[[1]],ddd[i],ddd[-i],n)
         }else{
-          stop('Wrong rank method !')  # should be error
+          stop('Wrong rank method !')
         }
         if(inputr==FALSE){
           r[i]=factor.num[i,penalty,1+iiter]
         }
-
+        
         eigen.gap[i,1:dd[i],1+iiter]=ans.iter$lambda[[1]]
       }
       #x.hat <- get.hat(x.tnsr,ans.Q,d.seq)
       x.hat <- ttl(x.tnsr,lapply(ans.Q,t),d.seq)
       x.hat <- ttl(x.hat,ans.Q,d.seq)
-
+      
       fnorm.resid[iiter+1] <- fnorm(x.tnsr-x.hat)/tnsr.norm
       dis <- abs(fnorm.resid[iiter+1] - fnorm.resid[iiter])
       if(iiter==1){
@@ -304,12 +306,12 @@ tenFM.rank = function(x,r,h0=1,rank='BIC',method='TIPUP',inputr=FALSE,iter=TRUE,
   }else{
     iiter <- iiter + 1
   }
-
+  
   factor.num[,,maxiter]=factor.num[,,iiter]
   eigen.gap[,,maxiter]=eigen.gap[,,iiter]
   fnorm.resid <- fnorm.resid[fnorm.resid != 0]
   norm.percent <- c(ans.init$norm.percent,fnorm.resid[1],tail(fnorm.resid,1))
-  return(list("path"=factor.num[,penalty,1:(iiter)],"factor.num"=factor.num[,penalty,maxiter]))
+  return(list("path"=t(factor.num[,penalty,1:(iiter)]),"factor.num"=factor.num[,penalty,maxiter]))
 }
 
 
@@ -401,11 +403,11 @@ tensor.bic<-function(reigen,h0=1,p1,p2,n,delta1=0){
   factor.p1=numeric(5)
   p=p1*p2
   m1=ceiling(p1/3)
-
+  
   lambda.p1<-reigen[p1:1]
   cumlambda.p1<-cumsum(lambda.p1)
   cumlambda.p1<-cumlambda.p1[(p1-1):1]
-
+  
   #threshold
   ic=cumlambda.p1[1:m1]/p^2*p^delta+(1:m1)*h0*(1/n)*log(p*n/(p+n))
   factor.p1[1]<-which.min(ic)
@@ -417,7 +419,7 @@ tensor.bic<-function(reigen,h0=1,p1,p2,n,delta1=0){
   factor.p1[4]<-which.min(ic)
   ic=cumlambda.p1[1:m1]/p^2*p^delta+(1:m1)*h0*(1/n+1/p)*log(min(p1,n))
   factor.p1[5]<-which.min(ic)
-
+  
   factor.p1
 }
 
@@ -428,11 +430,11 @@ tensor.ratio<-function(reigen,p1,p2,n){
   factor.p1=numeric(5)
   p=p1*p2
   m1=ceiling(p1/3)
-
+  
   lambda.p1<-reigen[p1:1]
   cumlambda.p1<-cumsum(lambda.p1)
   cumlambda.p1<-cumlambda.p1[(p1-1):1]
-
+  
   #ratio
   ratio<-lambda.p1[(p1-1):(p1-m1)]/lambda.p1[p1:(p1-m1+1)]
   factor.p1[1]<-which.min(ratio)
@@ -444,7 +446,7 @@ tensor.ratio<-function(reigen,p1,p2,n){
   factor.p1[4]<-which.min(ratio)
   ratio<-(lambda.p1[(p1-1):(p1-m1)] +p^2*(1/n^2/p1+1/n^2/p2))/(lambda.p1[p1:(p1-m1+1)] +p^2*(1/n^2/p1+1/n^2/p2))  #p^2*(1/n)*log(p*n/(p+n))
   factor.p1[5]<-which.min(ratio)
-
+  
   factor.p1
 }
 
@@ -498,7 +500,7 @@ topup.init.tensor <- function(x,r,h0=1,oneside.true=FALSE,norm.true=FALSE){
   # x: d1 * d2 * d3 * ... * d_d * n
   # if oneside.true==TRUE, then only compute the one sided column space,
   # not the other sides, this option is useful for the iterative method
-
+  
   dd <- dim(x)
   d <- length(dd) # d >= 2
   n <- dd[d]
@@ -535,9 +537,9 @@ topup.init.tensor <- function(x,r,h0=1,oneside.true=FALSE,norm.true=FALSE){
     #x.hat <- get.hat(x.tnsr,ans.Q,1:(d-1))
     x.hat <- ttl(x.tnsr,lapply(ans.Q,t),1:(d-1))
     x.hat <- ttl(x.hat,ans.Q,1:(d-1))
-
+    
     norm.percent <- fnorm(x.tnsr-x.hat)/fnorm(x.tnsr)
     x.hat <- x.hat@data
   }
   list("M"=ans.M,"Q"=ans.Q,"lambda"=ans.lambda,"norm.percent"=norm.percent,"x.hat"=x.hat)
-}
+}
\ No newline at end of file
